@@ -28,7 +28,7 @@ func Test_GetDocumentsByIDs(t *testing.T) {
 	ts, err := i.AddDocuments(request, nil)
 	require.NoError(t, err)
 
-	testWaitForTask(t, i, ts)
+	testWaitForIndexTask(t, i, ts)
 
 	var documents meilisearch.DocumentsResult
 	err = sv.Index("TestGetDocumentsByIDs").GetDocuments(&meilisearch.DocumentsQuery{Ids: []string{"1", "2", "3"}}, &documents)
@@ -63,12 +63,12 @@ func Test_GetDocumentsWithQuery(t *testing.T) {
 	index := sv.Index(indexUID)
 	task, err := index.AddDocuments(testDocuments, nil)
 	require.NoError(t, err)
-	testWaitForTask(t, index, task)
+	testWaitForIndexTask(t, index, task)
 
 	// Set sortable attributes for sorting tests
 	task, err = index.UpdateSortableAttributes(&[]string{"title", "rating", "year"})
 	require.NoError(t, err)
-	testWaitForTask(t, index, task)
+	testWaitForIndexTask(t, index, task)
 
 	tests := []struct {
 		name          string
@@ -318,7 +318,7 @@ func Test_AddOrUpdateDocumentsWithContentEncoding(t *testing.T) {
 			require.Equal(t, gotResp.IndexUID, "indexUID")
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			// Get Documents
 			var documents meilisearch.DocumentsResult
@@ -420,7 +420,7 @@ func TestIndex_AddOrUpdateDocuments(t *testing.T) {
 			require.Equal(t, tt.args.UID, gotResp.IndexUID)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var documents meilisearch.DocumentsResult
 			err = i.GetDocuments(&meilisearch.DocumentsQuery{Limit: 3}, &documents)
@@ -516,7 +516,7 @@ func TestIndex_AddDocumentsWithPrimaryKey(t *testing.T) {
 			i := c.Index(tt.args.UID)
 			t.Cleanup(cleanup(c))
 
-			gotResp, err := i.AddDocuments(tt.args.documentsPtr, &tt.args.primaryKey)
+			gotResp, err := i.AddDocuments(tt.args.documentsPtr, &meilisearch.DocumentOptions{PrimaryKey: &tt.args.primaryKey})
 			require.NoError(t, err)
 			require.GreaterOrEqual(t, gotResp.TaskUID, tt.resp.wantResp.TaskUID)
 			require.Equal(t, tt.resp.wantResp.Status, gotResp.Status)
@@ -524,7 +524,7 @@ func TestIndex_AddDocumentsWithPrimaryKey(t *testing.T) {
 			require.Equal(t, tt.args.UID, gotResp.IndexUID)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var documents meilisearch.DocumentsResult
 			err = i.GetDocuments(&meilisearch.DocumentsQuery{Limit: 3}, &documents)
@@ -668,7 +668,7 @@ func TestIndex_AddOrUpdateDocumentsInBatches(t *testing.T) {
 			i := c.Index(tt.args.UID)
 			t.Cleanup(cleanup(c))
 
-			gotResp, err := i.AddDocumentsInBatches(tt.args.documentsPtr, tt.args.batchSize, &tt.args.primaryKey)
+			gotResp, err := i.AddDocumentsInBatches(tt.args.documentsPtr, tt.args.batchSize, &meilisearch.DocumentOptions{PrimaryKey: &tt.args.primaryKey})
 
 			require.NoError(t, err)
 			for i := 0; i < 2; i++ {
@@ -761,7 +761,7 @@ func TestIndex_AddOrUpdateDocumentsNdjson(t *testing.T) {
 			require.Equal(t, tt.wantResp.Type, gotResp.Type)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var documents meilisearch.DocumentsResult
 			err = i.GetDocuments(&meilisearch.DocumentsQuery{}, &documents)
@@ -775,7 +775,7 @@ func TestIndex_AddOrUpdateDocumentsNdjson(t *testing.T) {
 				require.Equal(t, tt.wantResp.Status, gotResp.Status)
 				require.Equal(t, tt.wantResp.Type, gotResp.Type)
 				require.NotZero(t, gotResp.EnqueuedAt)
-				testWaitForTask(t, i, gotResp)
+				testWaitForIndexTask(t, i, gotResp)
 			}
 		})
 	}
@@ -966,7 +966,7 @@ func TestIndex_AddDocumentsCsv(t *testing.T) {
 			require.Equal(t, tt.wantResp.Type, gotResp.Type)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var documents meilisearch.DocumentsResult
 			err = i.GetDocuments(&meilisearch.DocumentsQuery{}, &documents)
@@ -1086,7 +1086,7 @@ func TestIndex_AddDocumentsCsvWithOptions(t *testing.T) {
 			require.Equal(t, tt.wantResp.Type, gotResp.Type)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var documents meilisearch.DocumentsResult
 			err = i.GetDocuments(&meilisearch.DocumentsQuery{}, &documents)
@@ -1261,14 +1261,14 @@ func TestIndex_DeleteAllDocuments(t *testing.T) {
 			t.Cleanup(cleanup(c))
 
 			setUpBasicIndex(tt.args.client, tt.args.UID)
-			gotResp, err := i.DeleteAllDocuments()
+			gotResp, err := i.DeleteAllDocuments(&meilisearch.DocumentOptions{})
 			require.NoError(t, err)
 			require.GreaterOrEqual(t, gotResp.TaskUID, tt.wantResp.TaskUID)
 			require.Equal(t, tt.wantResp.Status, gotResp.Status)
 			require.Equal(t, tt.wantResp.Type, gotResp.Type)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var documents meilisearch.DocumentsResult
 			err = i.GetDocuments(&meilisearch.DocumentsQuery{Limit: 5}, &documents)
@@ -1407,16 +1407,16 @@ func TestIndex_DeleteOneDocument(t *testing.T) {
 			require.GreaterOrEqual(t, gotAddResp.TaskUID, tt.wantResp.TaskUID)
 			require.NoError(t, err)
 
-			testWaitForTask(t, i, gotAddResp)
+			testWaitForIndexTask(t, i, gotAddResp)
 
-			gotResp, err := i.DeleteDocument(tt.args.identifier)
+			gotResp, err := i.DeleteDocument(tt.args.identifier, nil)
 			require.NoError(t, err)
 			require.GreaterOrEqual(t, gotResp.TaskUID, tt.wantResp.TaskUID)
 			require.Equal(t, tt.wantResp.Status, gotResp.Status)
 			require.Equal(t, tt.wantResp.Type, gotResp.Type)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var document []map[string]interface{}
 			err = i.GetDocument(tt.args.identifier, nil, &document)
@@ -1521,16 +1521,16 @@ func TestIndex_DeleteDocuments(t *testing.T) {
 			gotAddResp, err := i.AddDocuments(tt.args.documentsPtr, nil)
 			require.NoError(t, err)
 
-			testWaitForTask(t, i, gotAddResp)
+			testWaitForIndexTask(t, i, gotAddResp)
 
-			gotResp, err := i.DeleteDocuments(tt.args.identifier)
+			gotResp, err := i.DeleteDocuments(tt.args.identifier, nil)
 			require.NoError(t, err)
 			require.GreaterOrEqual(t, gotResp.TaskUID, tt.wantResp.TaskUID)
 			require.Equal(t, tt.wantResp.Status, gotResp.Status)
 			require.Equal(t, tt.wantResp.Type, gotResp.Type)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var document docTest
 			for _, identifier := range tt.args.identifier {
@@ -1713,22 +1713,22 @@ func TestIndex_DeleteDocumentsByFilter(t *testing.T) {
 			gotAddResp, err := i.AddDocuments(tt.args.documentsPtr, nil)
 			require.NoError(t, err)
 
-			testWaitForTask(t, i, gotAddResp)
+			testWaitForIndexTask(t, i, gotAddResp)
 
 			if len(tt.args.filterToApply) != 0 {
 				gotTask, err := i.UpdateFilterableAttributes(&tt.args.filterToApply)
 				require.NoError(t, err)
-				testWaitForTask(t, i, gotTask)
+				testWaitForIndexTask(t, i, gotTask)
 			}
 
-			gotResp, err := i.DeleteDocumentsByFilter(tt.args.filterToDelete)
+			gotResp, err := i.DeleteDocumentsByFilter(tt.args.filterToDelete, nil)
 			require.NoError(t, err)
 			require.GreaterOrEqual(t, gotResp.TaskUID, tt.wantResp.TaskUID)
 			require.Equal(t, tt.wantResp.Status, gotResp.Status)
 			require.Equal(t, tt.wantResp.Type, gotResp.Type)
 			require.NotZero(t, gotResp.EnqueuedAt)
 
-			testWaitForTask(t, i, gotResp)
+			testWaitForIndexTask(t, i, gotResp)
 
 			var documents meilisearch.DocumentsResult
 			err = i.GetDocuments(&meilisearch.DocumentsQuery{}, &documents)
@@ -1756,7 +1756,7 @@ func TestIndex_UpdateDocumentsByFunction(t *testing.T) {
 			Function: "doc.title = `✨ ${doc.title.to_upper()} ✨`",
 		})
 		require.NoError(t, err)
-		testWaitForTask(t, idx, task)
+		testWaitForIndexTask(t, idx, task)
 	})
 
 	t.Run("Test User-defined Context", func(t *testing.T) {
@@ -1767,6 +1767,179 @@ func TestIndex_UpdateDocumentsByFunction(t *testing.T) {
 			Function: "if doc.id >= context.idmax {\n\t\t    doc = ()\n\t\t  } else {\n\t\t\t  doc.title = `✨ ${doc.title} ✨`\n\t\t\t}",
 		})
 		require.NoError(t, err)
-		testWaitForTask(t, idx, task)
+		testWaitForIndexTask(t, idx, task)
+	})
+}
+func TestIndex_DocumentOperationsWithCustomMetadata(t *testing.T) {
+	sv := setup(t, "")
+	t.Cleanup(cleanup(sv))
+
+	// Setup a basic index
+	indexUID := "TestCustomMetadata"
+	_, err := sv.CreateIndex(&meilisearch.IndexConfig{Uid: indexUID, PrimaryKey: "id"})
+	require.NoError(t, err)
+	i := sv.Index(indexUID)
+
+	filterableAttributes := []interface{}{"id"}
+
+	task, err := i.UpdateFilterableAttributes(&filterableAttributes)
+	require.NoError(t, err)
+	testWaitForIndexTask(t, i, task)
+
+	// Define common test data
+	documents := []map[string]interface{}{
+		{"id": "1", "title": "Document 1"},
+		{"id": "2", "title": "Document 2"},
+	}
+
+	tests := []struct {
+		name           string
+		action         func(t *testing.T) *meilisearch.TaskInfo
+		expectMetadata string
+	}{
+		{
+			name: "AddDocuments with Metadata",
+			action: func(t *testing.T) *meilisearch.TaskInfo {
+				meta := "meta-add-docs"
+				task, err := i.AddDocuments(documents, &meilisearch.DocumentOptions{
+					TaskCustomMetadata: meta,
+				})
+				require.NoError(t, err)
+				return task
+			},
+			expectMetadata: "meta-add-docs",
+		},
+		{
+			name: "UpdateDocuments with Metadata",
+			action: func(t *testing.T) *meilisearch.TaskInfo {
+				meta := "meta-update-docs"
+				updateDocs := []map[string]interface{}{
+					{"id": "1", "title": "Updated Document 1"},
+				}
+				task, err := i.UpdateDocuments(updateDocs, &meilisearch.DocumentOptions{
+					TaskCustomMetadata: meta,
+				})
+				require.NoError(t, err)
+				return task
+			},
+			expectMetadata: "meta-update-docs",
+		},
+		{
+			name: "DeleteDocument (Single) with Metadata",
+			action: func(t *testing.T) *meilisearch.TaskInfo {
+				meta := "meta-delete-one"
+				task, err := i.DeleteDocument("1", &meilisearch.DocumentOptions{
+					TaskCustomMetadata: meta,
+				})
+				require.NoError(t, err)
+				return task
+			},
+			expectMetadata: "meta-delete-one",
+		},
+		{
+			name: "DeleteDocuments (Batch) with Metadata",
+			action: func(t *testing.T) *meilisearch.TaskInfo {
+				meta := "meta-delete-batch"
+				task, err := i.DeleteDocuments([]string{"2"}, &meilisearch.DocumentOptions{
+					TaskCustomMetadata: meta,
+				})
+				require.NoError(t, err)
+				return task
+			},
+			expectMetadata: "meta-delete-batch",
+		},
+		{
+			name: "AddDocumentsNdjson with Metadata",
+			action: func(t *testing.T) *meilisearch.TaskInfo {
+				meta := "meta-ndjson"
+				ndjson := []byte(`{"id": "3", "title": "Ndjson Doc"}`)
+				task, err := i.AddDocumentsNdjson(ndjson, &meilisearch.DocumentOptions{
+					TaskCustomMetadata: meta,
+				})
+				require.NoError(t, err)
+				return task
+			},
+			expectMetadata: "meta-ndjson",
+		},
+		{
+			name: "DeleteAllDocuments with Metadata",
+			action: func(t *testing.T) *meilisearch.TaskInfo {
+				meta := "meta-delete-all"
+				task, err := i.DeleteAllDocuments(&meilisearch.DocumentOptions{
+					TaskCustomMetadata: meta,
+				})
+				require.NoError(t, err)
+				return task
+			},
+			expectMetadata: "meta-delete-all",
+		},
+	}
+
+	// Run Standard Document Option Tests
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			// 1. Execute Action
+			taskInfo := tt.action(t)
+
+			// 2. Wait for task to be processed (ensures storage)
+			testWaitForIndexTask(t, i, taskInfo)
+
+			// 3. Fetch the full task details from the engine
+			task, err := sv.GetTask(taskInfo.TaskUID)
+			require.NoError(t, err)
+
+			// 4. Verify the metadata matches
+			require.Equal(t, tt.expectMetadata, task.CustomMetadata, "CustomMetadata should be persisted and retrievable from the Task object")
+		})
+	}
+
+	// Special Case: UpdateDocumentsByFunction
+	t.Run("UpdateDocumentsByFunction with Metadata", func(t *testing.T) {
+		// Ensure we have a doc to update
+		setupTask, _ := i.AddDocuments([]map[string]interface{}{{"id": "99", "title": "Function Doc"}}, nil)
+		testWaitForIndexTask(t, i, setupTask)
+
+		meta := "meta-function-update"
+
+		// Enable feature
+		exp := sv.ExperimentalFeatures()
+		exp.SetEditDocumentsByFunction(true)
+		_, err := exp.Update()
+		require.NoError(t, err)
+
+		// Perform Update
+		taskInfo, err := i.UpdateDocumentsByFunction(&meilisearch.UpdateDocumentByFunctionRequest{
+			Filter:             "id = 99",
+			Function:           "doc.title = \"Updated Function\"",
+			TaskCustomMetadata: meta,
+		})
+		require.NoError(t, err)
+
+		// Wait and Verify
+		testWaitForIndexTask(t, i, taskInfo)
+
+		task, err := sv.GetTask(taskInfo.TaskUID)
+		require.NoError(t, err)
+		require.Equal(t, meta, task.CustomMetadata)
+	})
+
+	// Special Case: DeleteDocumentsByFilter
+	t.Run("DeleteDocumentsByFilter with Metadata", func(t *testing.T) {
+		// Ensure we have a doc to delete
+		setupTask, _ := i.AddDocuments([]map[string]interface{}{{"id": "99", "title": "Filter Doc"}}, nil)
+		testWaitForIndexTask(t, i, setupTask)
+
+		meta := "meta-delete-filter"
+		// Note: "id" was made filterable at the top of the test function
+		taskInfo, err := i.DeleteDocumentsByFilter("id = 99", &meilisearch.DocumentOptions{
+			TaskCustomMetadata: meta,
+		})
+		require.NoError(t, err)
+
+		testWaitForIndexTask(t, i, taskInfo)
+
+		task, err := sv.GetTask(taskInfo.TaskUID)
+		require.NoError(t, err)
+		require.Equal(t, meta, task.CustomMetadata)
 	})
 }
