@@ -15,15 +15,31 @@ const (
 )
 
 // Network represents the Meilisearch network configuration.
-// Each field is wrapped in an Opt so it can be explicitly included,
-// set to JSON null, or omitted entirely.
 type Network struct {
-	Self     Opt[string]                 `json:"self,omitempty"`
-	Remotes  Opt[map[string]Opt[Remote]] `json:"remotes,omitempty"`
-	Sharding Opt[bool]                   `json:"sharding,omitempty"`
+	Self    string            `json:"self,omitempty"`
+	Leader  string            `json:"leader,omitempty"`
+	Remotes map[string]Remote `json:"remotes,omitempty"`
+	Version string            `json:"version,omitempty"`
 }
 
-func (n Network) MarshalJSON() ([]byte, error) {
+// Remote describes a single remote Meilisearch node.
+type Remote struct {
+	URL          string `json:"url"`
+	SearchAPIKey string `json:"searchApiKey"`
+	WriteAPIKey  string `json:"writeApiKey"`
+}
+
+// UpdateNetworkRequest represents the Meilisearch network configuration update without leader.
+// Each field is wrapped in an Opt so it can be explicitly included,
+// set to JSON null, or omitted entirely.
+type UpdateNetworkRequest struct {
+	Self    Opt[string]                       `json:"self,omitempty"`
+	Leader  Opt[string]                       `json:"leader,omitempty"`
+	Remotes Opt[map[string]Opt[UpdateRemote]] `json:"remotes,omitempty"`
+	Version Opt[string]                       `json:"version,omitempty"`
+}
+
+func (n UpdateNetworkRequest) MarshalJSON() ([]byte, error) {
 	m := make(map[string]any)
 
 	if n.Self.Valid() {
@@ -38,25 +54,31 @@ func (n Network) MarshalJSON() ([]byte, error) {
 		m["remotes"] = nil
 	}
 
-	if n.Sharding.Valid() {
-		m["sharding"] = n.Sharding.Value
-	} else if n.Sharding.Null() {
-		m["sharding"] = nil
+	if n.Leader.Valid() {
+		m["leader"] = n.Leader.Value
+	} else if n.Leader.Null() {
+		m["leader"] = nil
+	}
+
+	if n.Version.Valid() {
+		m["version"] = n.Version.Value
+	} else if n.Version.Null() {
+		m["version"] = nil
 	}
 
 	return json.Marshal(m)
 }
 
-// Remote describes a single remote Meilisearch node.
+// UpdateRemote describes a single remote Meilisearch node.
 // Each field is wrapped in an Opt so it can be explicitly included,
 // set to JSON null, or omitted entirely.
-type Remote struct {
+type UpdateRemote struct {
 	URL          Opt[string] `json:"url"`
 	SearchAPIKey Opt[string] `json:"searchApiKey,omitempty"`
 	WriteAPIKey  Opt[string] `json:"writeApiKey,omitempty"`
 }
 
-func (r Remote) MarshalJSON() ([]byte, error) {
+func (r UpdateRemote) MarshalJSON() ([]byte, error) {
 	m := make(map[string]any)
 
 	if r.URL.Valid() {
@@ -301,19 +323,20 @@ type Stats struct {
 //
 // Documentation: https://www.meilisearch.com/docs/learn/advanced/asynchronous_operations
 type Task struct {
-	Status      TaskStatus          `json:"status"`
-	UID         int64               `json:"uid,omitempty"`
-	TaskUID     int64               `json:"taskUid,omitempty"`
-	IndexUID    string              `json:"indexUid"`
-	Type        TaskType            `json:"type"`
-	Error       meilisearchApiError `json:"error,omitempty"`
-	TaskNetwork TaskNetwork         `json:"network,omitempty"`
-	Duration    string              `json:"duration,omitempty"`
-	EnqueuedAt  time.Time           `json:"enqueuedAt"`
-	StartedAt   time.Time           `json:"startedAt,omitempty"`
-	FinishedAt  time.Time           `json:"finishedAt,omitempty"`
-	Details     Details             `json:"details,omitempty"`
-	CanceledBy  int64               `json:"canceledBy,omitempty"`
+	Status         TaskStatus          `json:"status"`
+	UID            int64               `json:"uid,omitempty"`
+	TaskUID        int64               `json:"taskUid,omitempty"`
+	IndexUID       string              `json:"indexUid"`
+	Type           TaskType            `json:"type"`
+	Error          meilisearchApiError `json:"error,omitempty"`
+	TaskNetwork    TaskNetwork         `json:"network,omitempty"`
+	Duration       string              `json:"duration,omitempty"`
+	EnqueuedAt     time.Time           `json:"enqueuedAt"`
+	StartedAt      time.Time           `json:"startedAt,omitempty"`
+	FinishedAt     time.Time           `json:"finishedAt,omitempty"`
+	Details        Details             `json:"details,omitempty"`
+	CanceledBy     int64               `json:"canceledBy,omitempty"`
+	CustomMetadata string              `json:"customMetadata,omitempty"`
 }
 
 // TaskNetwork indicates information about a task network
@@ -651,9 +674,23 @@ type SimilarDocumentResult struct {
 	EstimatedTotalHits int64  `json:"estimatedTotalHits,omitempty"`
 }
 
+// DocumentOptions is the options struct for adding or updating documents (JSON/NDJSON)
+// and deleting documents.
+type DocumentOptions struct {
+	PrimaryKey *string `json:"primaryKey,omitempty"`
+	// TaskCustomMetadata is the custom metadata to add to the task.
+	// This string will be associated with the task and visible in the task details.
+	// It is optional.
+	TaskCustomMetadata string `json:"-"`
+}
+
 type CsvDocumentsQuery struct {
 	PrimaryKey   string `json:"primaryKey,omitempty"`
 	CsvDelimiter string `json:"csvDelimiter,omitempty"`
+	// TaskCustomMetadata is the custom metadata to add to the task.
+	// This string will be associated with the task and visible in the task details.
+	// It is optional.
+	TaskCustomMetadata string `json:"-"`
 }
 
 type DocumentsResult struct {
@@ -667,6 +704,10 @@ type UpdateDocumentByFunctionRequest struct {
 	Filter   string                 `json:"filter,omitempty"`
 	Function string                 `json:"function"`
 	Context  map[string]interface{} `json:"context,omitempty"`
+	// TaskCustomMetadata is the custom metadata to add to the task.
+	// This string will be associated with the task and visible in the task details.
+	// It is optional.
+	TaskCustomMetadata string `json:"-"`
 }
 
 // ExperimentalFeaturesResult represents the experimental features result from the API.
